package com.vendor.burn.thread;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.analysis.TimeLogger;
import com.vendor.burn.consoles.FastbootConsoleFactory;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.receive.ReceiveTimeOutException;
import com.vendor.burn.serial.workers.EmmcPartitionBurnerWorker;
import com.vendor.burn.sparse.SparseFileAgent;
import com.vendor.burn.sparse.SparseHeader;
import com.vendor.burn.thread.PartitionTempObj;
import com.vendor.burn.thread.TempFileInfo;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.GZipUtils;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;

public class LargeFileTogetherThread {
  protected static final int SECTOR_SIZE = 512;
  
  private static final int FILE_BLOCK_SIZE_32 = 33554432;
  
  private boolean stopFlag;
  
  private ArrayList<PartitionTempObj> partitionTempObjList;
  
  private EmmcPartitionBurnerWorker emmcPartitionBurnerWorker;
  
  private boolean tftpOccupied;
  
  public LargeFileTogetherThread(EmmcPartitionBurnerWorker emmcPartitionBurnerWorker) {
    this.emmcPartitionBurnerWorker = emmcPartitionBurnerWorker;
    this.partitionTempObjList = new ArrayList<>();
    this.partitionTempObjList.clear();
  }
  
  public void stop() {
    this.stopFlag = true;
  }
  
  public void startMakeZipFile(IProgressMonitor uiMonitor, List<PartitionItem> parts, long tempFileDivisionSize, IFastbootProgressMonitor monitor) throws IOException, ReceiveTimeOutException {
    this
      .tftpOccupied = this.emmcPartitionBurnerWorker.getBurner().getBurnContext().isTftpOccupied();
    TimeLogger zipAndPageTimeLogger = this.emmcPartitionBurnerWorker.getZipAndPageTimeLogger();
    this.stopFlag = false;
    for (int partIndex = 0; partIndex < parts.size(); partIndex++) {
      if (uiMonitor.isCanceled()) {
        clearTempFile();
        break;
      } 
      if (this.stopFlag) {
        clearTempFile();
        break;
      } 
      PartitionTempObj partitionTempObj = new PartitionTempObj();
      PartitionItem partitionItem = parts.get(partIndex);
      if (partitionItem.isChecked()) {
        File file = partitionItem.getFile();
        if (file != null && file.exists()) {
          long filePartSize = file.length();
          if (filePartSize >= 33554432L) {
            FileSystem fileSystem = partitionItem.getFileSystem();
            String partitionName = partitionItem.getName();
            if (partitionName == null)
              partitionName = ""; 
            zipAndPageTimeLogger.logStart(partitionName, partitionItem
                .getFile().length());
            int totalTempCount = (int)((filePartSize + tempFileDivisionSize - 1L) / tempFileDivisionSize);
            partitionTempObj.setPartitionName(partitionName);
            partitionTempObj.setTotalTempCount(totalTempCount);
            boolean isSparse = (SparseFileAgent.isSparse(fileSystem) && SparseFileAgent.isAndroidSparseFile(file));
            boolean result = true;
            partitionTempObj.setNeedWaitZip(true);
            if (filePartSize > tempFileDivisionSize) {
              this.emmcPartitionBurnerWorker.getSpellingSparseFileLogger()
                .logStart(file.getName(), filePartSize);
              if (isSparse) {
                result = createSparseTempFile(uiMonitor, partitionItem, partitionTempObj, tempFileDivisionSize, monitor);
              } else {
                result = createTempFile(uiMonitor, partIndex, file, tempFileDivisionSize, result, partitionTempObj, monitor);
              } 
              this.emmcPartitionBurnerWorker.getSpellingSparseFileLogger()
                .logEnd();
            } else {
              String partFilePath = null;
              if (this.tftpOccupied) {
                partFilePath = file.getParent() + file.getParent() + File.separator + partIndex;
              } else {
                partFilePath = ToolUtils.getInstallPath() + ToolUtils.getInstallPath() + File.separator + partIndex;
              } 
              File partFile = new File(partFilePath);
              boolean copy = FileUtils.copy(uiMonitor, file, partFile);
              if (!copy) {
                monitor.println(NLS.bind(Messages.ERROR_INFO_DISK_SPACE_LOW, partFile));
                Display.getDefault().asyncExec((Runnable)new Object(this, partFile));
                return;
              } 
              if (isSparse) {
                RandomAccessFile inFile = null;
                try {
                  inFile = new RandomAccessFile(file, "r");
                  byte[] sparseHeaderData = new byte[28];
                  inFile.seek(0L);
                  int read = inFile.read(sparseHeaderData);
                  if (read < 0)
                    System.out.println("Read Error"); 
                  SparseHeader sparseHeader = new SparseHeader();
                  sparseHeader.fromBytes(sparseHeaderData);
                  long tempFileBurnNeedSize = sparseHeader.blk_sz * sparseHeader.total_blks;
                  File zipTempFile = GZipUtils.compress(uiMonitor, partFile, false, this.tftpOccupied);
                  if (zipTempFile == null)
                    return; 
                  if (partFile != null && partFile.exists())
                    FileUtils.deleteFile(partFile); 
                  TempFileInfo tempFileInfo = new TempFileInfo();
                  tempFileInfo.setZipTempFile(zipTempFile);
                  tempFileInfo
                    .setTempFileBurnNeedSize(tempFileBurnNeedSize);
                  tempFileInfo.setStartAddressOffset(0L);
                  tempFileInfo.setBurnState(false);
                  partitionTempObj.getTempFileInfoList()
                    .add(tempFileInfo);
                  if (!this.partitionTempObjList.contains(partitionTempObj))
                    this.partitionTempObjList.add(partitionTempObj); 
                } catch (Exception exception) {
                
                } finally {
                  if (inFile != null)
                    inFile.close(); 
                } 
              } else {
                File zipTempFile = GZipUtils.compress(uiMonitor, partFile, false, this.tftpOccupied);
                if (zipTempFile == null)
                  return; 
                TempFileInfo tempFileInfo = new TempFileInfo();
                tempFileInfo.setZipTempFile(zipTempFile);
                tempFileInfo.setTempFileBurnNeedSize(file.length());
                tempFileInfo.setStartAddressOffset(0L);
                tempFileInfo.setBurnState(false);
                partitionTempObj.getTempFileInfoList().add(tempFileInfo);
                if (!this.partitionTempObjList.contains(partitionTempObj))
                  this.partitionTempObjList.add(partitionTempObj); 
              } 
              if (partFile != null && partFile.exists()) {
                boolean isDelete = FileUtils.deleteFile(partFile);
                if (!isDelete)
                  monitor.println(NLS.bind(Messages.FileDeleteFailed, partFile)); 
              } 
            } 
            partitionTempObj.setNeedWaitZip(false);
            zipAndPageTimeLogger.logEnd();
          } 
        } 
      } 
    } 
  }
  
  public long getTempFileSize(long ddrFreeSize) {
    long blockNumber = ddrFreeSize / 33554432L;
    if (blockNumber <= 0L)
      blockNumber = 1L; 
    return 33554432L * blockNumber;
  }
  
  public void clearTempFile() {
    for (int i = 0; i < this.partitionTempObjList.size(); i++) {
      PartitionTempObj partitionTempObj = this.partitionTempObjList.get(i);
      List<TempFileInfo> tempFileInfoList = partitionTempObj.getTempFileInfoList();
      for (int j = 0; j < tempFileInfoList.size(); j++) {
        TempFileInfo tempFileInfo = tempFileInfoList.get(j);
        if (tempFileInfo != null) {
          File zipTempFile = tempFileInfo.getZipTempFile();
          if (zipTempFile.exists())
            deleteTempFile(zipTempFile); 
        } 
      } 
    } 
  }
  
  private void deleteTempFile(File file) {
    long startTime = System.currentTimeMillis();
    long outTime = 5000L;
    while (file != null && !file.delete() && 
      System.currentTimeMillis() - startTime < outTime) {
      try {
        Thread.sleep(1L);
      } catch (InterruptedException interruptedException) {}
    } 
    if (file != null && file.exists())
      FastbootConsoleFactory.getDebugConsole()
        .getWarningStream()
        .println(file
          .getName() + " failed to automatically deleted, please removed manually!"); 
  }
  
  protected boolean createTempFile(IProgressMonitor uiMonitor, int partIndex, File file, long tempFileDivisionSize, boolean result, PartitionTempObj partitionTempObj, IFastbootProgressMonitor monitor) throws FileNotFoundException, IOException {
    FileInputStream inputStream = null;
    int tempTimes = 0;
    try {
      if (file != null && file.exists() && file.isFile()) {
        inputStream = new FileInputStream(file);
        byte[] buffer = new byte[2097152];
        long totalLength = file.length();
        long startAddrOffset = 0L;
        while (result && totalLength > 0L) {
          File partfile = null;
          int partLength = 0;
          try {
            int read = -1;
            boolean tftpOccupied = this.emmcPartitionBurnerWorker.getBurner().getBurnContext().isTftpOccupied();
            String partFilePath = null;
            if (tftpOccupied) {
              partFilePath = file.getParent() + file.getParent() + File.separator + ".part" + file.getName();
            } else {
              partFilePath = ToolUtils.getInstallPath() + ToolUtils.getInstallPath() + File.separator + partIndex + ".part" + file.getName();
            } 
            partfile = new File(partFilePath);
            boolean createNewFile = partfile.createNewFile();
            if (!createNewFile)
              System.out.println("Failed to create file " + partfile
                  .getName()); 
            OutputStream outputtream = null;
            try {
              outputtream = new FileOutputStream(partfile);
              int length = (int)Math.min(tempFileDivisionSize, buffer.length);
              while (partLength < tempFileDivisionSize && result && (
                
                read = inputStream.read(buffer, 0, length)) > 0) {
                outputtream.write(buffer, 0, read);
                partLength += read;
                if (tempFileDivisionSize - partLength < buffer.length) {
                  length = (int)(tempFileDivisionSize - partLength);
                  continue;
                } 
                length = buffer.length;
              } 
            } finally {
              if (outputtream != null)
                outputtream.close(); 
            } 
            File zipTempFile = GZipUtils.compress(uiMonitor, partfile, false, tftpOccupied);
            if (zipTempFile == null)
              return false; 
            TempFileInfo tempFileInfo = new TempFileInfo();
            tempFileInfo.setZipTempFile(zipTempFile);
            tempFileInfo.setStartAddressOffset(startAddrOffset);
            tempFileInfo.setTempFileBurnNeedSize(partLength);
            tempFileInfo.setBurnState(false);
            partitionTempObj.getTempFileInfoList()
              .add(tempFileInfo);
            if (!this.partitionTempObjList.contains(partitionTempObj))
              this.partitionTempObjList.add(partitionTempObj); 
            startAddrOffset += partLength;
            totalLength -= partLength;
          } finally {
            if (partfile != null && partfile.exists())
              FileUtils.deleteFile(partfile); 
          } 
        } 
      } 
    } finally {
      partitionTempObj.setTotalTempCount(partitionTempObj
          .getTempFileInfoList().size());
      if (inputStream != null)
        inputStream.close(); 
    } 
    return result;
  }
  
  private boolean createSparseTempFile(IProgressMonitor uiMonitor, PartitionItem partitionItem, PartitionTempObj partitionTempObj, long ddrUsableSize, IFastbootProgressMonitor monitor) throws FileNotFoundException, IOException {
    List<PartitionItem> partitionItemList = SparseFileAgent.getSparseFileXmlList(ddrUsableSize, partitionItem, this.tftpOccupied);
    if (partitionItemList.size() == 0)
      return false; 
    File tmpFile = null;
    long offset = 0L;
    try {
      for (int i = 0; i < partitionItemList.size(); i++) {
        PartitionItem item = partitionItemList.get(i);
        tmpFile = item.getFile();
        File zipTempFile = GZipUtils.compress(uiMonitor, tmpFile, false, this.tftpOccupied);
        if (zipTempFile == null)
          return false; 
        long fileBurnNeedSize = item.getLength();
        TempFileInfo tempFileInfo = new TempFileInfo();
        tempFileInfo.setZipTempFile(zipTempFile);
        tempFileInfo
          .setTempFileBurnNeedSize(fileBurnNeedSize);
        tempFileInfo.setStartAddressOffset(offset);
        tempFileInfo.setBurnState(false);
        partitionTempObj.getTempFileInfoList().add(tempFileInfo);
        offset += fileBurnNeedSize;
        if (!this.partitionTempObjList.contains(partitionTempObj))
          this.partitionTempObjList.add(partitionTempObj); 
        if (!FileUtils.deleteFile(tmpFile))
          monitor.println(NLS.bind(Messages.FileDeleteFailed, tmpFile)); 
      } 
    } finally {
      partitionTempObj.setTotalTempCount(partitionTempObj
          .getTempFileInfoList().size());
      if (!FileUtils.deleteFile(tmpFile))
        monitor.println(NLS.bind(Messages.FileDeleteFailed, tmpFile)); 
    } 
    return true;
  }
  
  protected File createTempFile(int partIndex, File file, int tempTimes) {
    File tmpFile = null;
    String toolPath = null;
    if (this.tftpOccupied) {
      try {
        tmpFile = new File(file.getCanonicalPath() + ".tmp" + file.getCanonicalPath());
      } catch (IOException e) {
        return tmpFile;
      } 
    } else {
      toolPath = ToolUtils.getInstallPath();
      if (toolPath.endsWith(File.separator)) {
        tmpFile = new File(toolPath + toolPath + partIndex + ".tmp" + file.getName());
      } else {
        tmpFile = new File(toolPath + toolPath + File.separator + partIndex + ".tmp" + file.getName());
      } 
    } 
    return tmpFile;
  }
  
  public ArrayList<PartitionTempObj> getPartitionTempObjList() {
    return this.partitionTempObjList;
  }
  
  public void setPartitionTempObjList(ArrayList<PartitionTempObj> partitionTempObjList) {
    this.partitionTempObjList = partitionTempObjList;
  }
}
